<?php

namespace Avos\Utils;

function object2array($data) {

    if (is_object($data)) {
        $data = get_object_vars($data);
    }

    if (is_array($data)) {
        return array_map(__FUNCTION__, $data);
    }
    else {
        return $data;
    }
}

function parseTemplate($template, $args=array())
{
    global $sitename, $main_sitename, $selfurl;

    foreach ($args as $k  => $v) {
        $$k = $v;
    }

    $TOP = "";

    ob_start();
    include("templates/" . $template);
    $html = ob_get_contents();
    ob_end_clean();

    return $html;
}

function urlsafe($title, $char = '-')
{
	$tmp = '1234567890qwertyuiop0987654321';

	$title = preg_replace('/\s+/', $tmp, $title);
	$title = preg_replace('/\W/', $tmp, $title);
	$title = preg_replace("/($tmp)+/", $char, $title);
	$title = trim($title, $char);

	return $title;
}

function seconds2string($seconds, $secs=true, $readable=false)
{
	$hours = floor($seconds / 3600);
	$minutes = floor(($seconds / 60) % 60);
	$seconds = $seconds % 60;

	if ($readable == false) {
		$minutes = $minutes < 10 ? '0' . $minutes : $minutes;
		$seconds = $seconds < 10 ? '0' . $seconds : $seconds;
	}

	if ($readable == false) {
		return "$hours:$minutes:$seconds";
	}
	else {
		$ret = "";
		if ($hours > 0) $ret .= "$hours hrs ";
		if ($minutes > 0) $ret .= "$minutes mins ";
		if ($ret == "" || $secs == true) $ret .= "$seconds secs";
		return $ret;
	}
}

function ordinal($number) 
{
	$ends = array('th','st','nd','rd','th','th','th','th','th','th');

	if ((($number % 100) >= 11) && (($number%100) <= 13)) 
		return $number. 'th';
	else
		return $number. $ends[$number % 10];
}

function monthName($i)
{
	$months = array('', 'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');
	return $months[(int)$i];
}

function NormaliseHistoryData($h)
{
	$ret = array();
	$webcamsId = $h[0]['webcamsId'];
	$epoch = $h[0]['epochTime'];
	$F = date('F', $epoch);
	$Y = date('Y', $epoch);

	$daysinmonth = $d=cal_days_in_month(CAL_GREGORIAN, date('n', $epoch), date('Y', $epoch));

	//
	// see what days we have data for
	//

	$days = array();

	foreach ($h as $e) {
		$days[ date('j', $e['epochTime']) ] = 1;
	}

	//
	// append missing days before start of month
	//

	for ($j=1; $j<date('j', $epoch); $j++) {
		$ret[] = array(
			'webcamsId' => $webcamsId, 
			'epochTime' => strtotime("$F $j, $Y, 12:00 am"),
			'status' => 'o', 
			'statusTime' => 0
		);
	}

	//
	// create new array from history data adding in data for missing days
	//

	$cday = 1;

	foreach ($h as $i => $e) {

		$ed = date('j', $e['epochTime']);
		$edend_epoch = strtotime("$F $ed, $Y, 12:00 am") + 60*60*24;

		#if ($ed != 25) continue; //fish

		//
		// append more data for current day or next day
		//

		if ($ed == $cday || $ed == $cday+1) {

			//
			// check to see if the data overlaps the next one
			//

			#echo $e['status'] . "/". $e['epochTime'] ."/". $e['statusTime'] . "<br>";

			/*
			if (isset($h[$i+1])) {
				if ($e['epochTime'] + $e['statusTime']/10 >= $h[$i+1]['epochTime']) {
					$e['statusTime'] = $h[$i+1]['epochTime'] - $e['epochTime'] - 1;
					echo "/overlap";
				}
			}
			*/

			/*
			if ($e['statusTime'] > 0) {
				$ret[] = $e;
			}

			$cday = $ed;
			*/

			/************* NOT WORKING - going over midnight ************/

			if ($e['epochTime'] + $e['statusTime'] < $edend_epoch) {
				$ret[] = $e;
				$cday = $ed;
			}
			else {
				$pre = $e['epochTime'] + $e['statusTime'] - $edend_epoch;
				$post = $e['statusTime'] - $pre;

				$e['statusTime'] = $pre;
				$ret[] = $e;

				$e['epochTime'] = $edend_epoch;
				$e['statusTime'] = $post;

				$cday = $ed+1;
				$days[$cday] = 1;
			}
		}

		//
		// append missing days then the data
		//

		else {

			for ($j=$cday+1; $j<$ed; $j++) {
				$ret[] = array(
					'webcamsId' => $webcamsId, 
					'epochTime' => strtotime("$F $j, $Y, 12:00 am"),
					'status' => 'o', 
					'statusTime' => 0
				);
			}

			$ret[] = $e;
			$cday = $j;
		}
	}

	//
	// append missing days before end of month
	//

	for ($j=$cday+1; $j<=$daysinmonth; $j++) {
		$ret[] = array(
			'webcamsId' => $webcamsId, 
			'epochTime' => strtotime("$F $j, $Y, 12:00 am"),
			'status' => 'o', 
			'statusTime' => 0
		);
	}

	/*
	foreach ($ret as $i => $r) {
		if ($r['status'] != 'offline') {
			echo "$i " . $r['status'] . " == " . date('l jS \of F Y h:i:s A', $r['epochTime']) . "   " . $r['statusTime'] . "<br>";
		}
	}
	*/

	return $ret;
}

function imagePath($m)
{
	global $_AVOS;

	$m['nickname'] = strtolower($m['nickname']);

    if (preg_match('!^(.)(.)(.)(.)!', $m['nickname'], $c)) {

		$site = strtolower(\Avos\Scaffold\expandSiteName($m['site']));
		$top = $_AVOS['config']['webcams']['imagesUrl'];
        return "$top$site/$c[1]/$c[2]/$c[3]/$c[4]/$m[nickname]/";

	}
    else if (preg_match('!^(.)(.)(.)!', $m['nickname'], $c)) {

        // seriouslsy... why would a model have a nickname with less than 4 letters in???
        // use 'xx' since all folders are single letters usually

		$site = strtolower(\Avos\Scaffold\expandSiteName($m['site']));
		$top = $_AVOS['config']['webcams']['imagesUrl'];
		return "$top$site/$c[1]/$c[2]/$c[3]/xx/$m[nickname]/";
	}
	else {
		return '';
	}
}

function query2sql($q, $expand_categories=false)
{
    if (is_array($q)) {
        $q = array2url($q);
    }

    #echo "query2sql> $q\n";

    $qa = explode('||', $q);
    $sql = array();

    foreach ($qa as $qaq) {
        $sql[] = query2sql_aux($qaq, $expand_categories);
    }

    if (count($sql) == 1) {
        $r =  $sql[0];
    }
    else {
        $r = '(' . implode(') OR (', $sql) . ')';
    }

    #echo "query2sql< $r\n";
    return $r;
}

function query2sql_aux($q, $expand_categories=false)
{
	global $_AVOS;
	$sql = array();
    $redis = $_AVOS['redis'];

    #echo "aux> $q\n";

	//
	// make sure we have an array
	//

	if (!is_array($q)) {
		$q = parse_str($q, $qq);
		$q = $qq;
    }

    //
    // delete sort=
    //

    unset($q['sort']);

	//
	// set sites filter if defined
	//

	$sitesFilter = str_replace(',', '|', \getConfig('webcams', 'sitesFilter', ''));

	//
	// if we have a key of 'category' or 'tag' then expand filters
	//

    $dbg = DEBUG;
    if ($dbg) { 
        echo "<!--\n";
        foreach ($q as $k => $v) {
            echo "[$k] => $v ";
        }
        echo "\n";
    }

    if (isset($_AVOS['page']['type']) && $_AVOS['page']['type'] == 'webcams') {

        //
        // if we have a 'category' or 'tag' and set to use either of the webcamsCategories or webcamsTags
        // tables (.e: $expand_categories is false) and don't have a 'status' set then add in status=!o to 
        // fetch more up to date status info from the webcams table
        //

        if ((isset($q['category']) || isset($q['tag'])) && !$expand_categories && !isset($q['status'])) {
            $q['status'] = '!o';
        }

        $categoriesPrefix = getConfig('redis', 'categoriesPrefix', '');
        if (!$categoriesPrefix) {

            //
            // standalone categories site
            //

            foreach (array('category', 'tag') as $single) {

                if ($single == 'category') {
                    $multi = 'Categories';
                    $multiLc = 'categories';
                }
                else {
                    $multi = 'Tags';
                    $multiLc = 'tags';
                }

                if (isset($q[$single])) {
                    $category = $q[$single];

                    if ($expand_categories) {

                        //
                        // we only only get here from cron to populate webcamsCategories/webcamsTags from filters
                        //

                        if (isset($_AVOS['config']['webcams'][$multiLc])) {
                            foreach ($_AVOS['config']['webcams'][$multiLc] as $n => $c) {
                                $c = trim($c, "[] ");
                                $cc = explode(',', $c);
	
                                if ($cc[0] == $category) {
                                    #if ($dbg) { echo "cc= "; print_r($cc);}
	
                                    // save for pagination
                                    $_AVOS['page'][$single] = $cc[0];
                                    $_AVOS['page']['title'] = $n;

                                    // add in siteFilter if set
                                    $filter = explode('||', $cc[1]);

                                    if (!empty($sitesFilter)) {
                                        foreach ($filter as $i => $f) {
                                            if (strpos($f, 'site=') == false) {
                                                $filter[$i] .= '&site=' . $sitesFilter;
                                            }
                                        }
                                    }

                                    // add in url extraArgs if set
                                    $urlName = 'webcams' . ucfirst($single);
                                    $urlValue = getConfig('urls', $urlName, '');

                                    if (!empty($urlValue)) {
                                        $urlTemp = explode(',', $urlValue);

                                        if (!empty($urlTemp[2])) {
                                            $extraArgs = rtrim($urlTemp[2], ']');
                                            foreach ($filter as $i => $f) {
                                                $filter[$i] .= '&' . $extraArgs;
                                            }
                                        }
                                    }

                                    $filter = implode('||', $filter);

                                    // since we now have a raw filter we can recurse
                                    return query2sql_aux($cc[1]);

                                    /*
                                    foreach (explode('&', $cc[1]) as $ccc) {
                                        if ($dbg) echo "$ccc  ";
                                        if (strpos($ccc, '=')) {
                                            $cccc = explode('=', $ccc);
                                            $q[ $cccc[0] ] = $cccc[1];
                                        }
                                        else {
                                            $q[ $ccc ] = '';
                                        }
                                    }
                                    if ($dbg) echo "\n";
                                     */
                                }
                            }
						}
					}
                    else {

                        //
                        // use the webcamsCategories table for site not using shared cats
                        //

                        $q[$single] = "id IN (SELECT webcamsId FROM webcams$multi WHERE name='$category')";
                    }
                }
            }
        }
        else {

            $tag = '';
            $where = [];

            $webcamsDatabase = getConfig('mysql', 'webcamsDatabase', '');
            $category = listWebcamsCategories('categories', true);
            $tag = listWebcamsCategories('tags', true);

            foreach (['category', 'tag'] as $single) {

                if ($single == 'category') {
                    $single2 = 'tag';
                    $multi = 'Categories';
                    $multiLc = 'categories';
                }
                else {
                    $single2 = 'category';
                    $multi = 'Tags';
                    $multiLc = 'tags';
                }

                if (isset($q[$single])) {

                    $c = $q[$single];
                    $ct = 'webcams' . $multi;

                    //
                    // $$single references either the array $category or $tag
                    //

                    if (!empty($$single[$c]['gender'])) {
                        $q['gender'] = implode('|', str_split($$single[$c]['gender']));
                    }

                    $c = isset($$single[$c]) ? $$single[$c]['filter'] : $c;

                    //
                    // we can check if data exists by looking in redis and switch between tags/cats if needed
                    //

                    if ($redis->type("$categoriesPrefix/webcams/$single/$c") == \Redis::REDIS_NOT_FOUND) {
                        $ct = $ct == 'webcamsCategories' ? 'webcamsTags' : 'webcamsCategories';
                    }

                    /*
                     * this alternative code would use model id's in place in the sql

                    $models = $redis->get("$categoriesPrefix/webcams/$single/$c");

                    if (is_array($models)) {
                        $ids = array_map(function($m) { return $m['i']; }, $models);
                        $where[] = "id IN (" . implode(',', $ids) . ")";
                    }
                     */

                    $where[] = "id IN (SELECT webcamsId FROM $webcamsDatabase.$ct WHERE name='$c')";
                }
            }

            if (count($where)) {
                $q['category'] = implode(' AND ', $where);
                unset($q['tag']);
            }

            /*
            $p = getConfig('redis', 'categoriesPrefix', '');

            $c = listWebcamsCategories($multiLc, true);
            $s = isset($c[$category]) ? $c[$category]['filter'] : $category;
            $r = \Avos\Redis\get("^$p/webcams/$single/$s");

            if (!$r) {
                //
                // if tag check for category and vice versa
                //

                $single = $single == 'category' ? 'tag' : 'category';
                $multiLc = $multiLc == 'categories' ? 'tags' : 'categories';

                $c = listWebcamsCategories($multiLc, true);

                if (isset($c[$category])) {
                    $s = $c[$category]['filter'];
                }

                $r = \Avos\Redis\get("^$p/webcams/$single/$s");
            }

            if ($p) {
                $shared = $p . '.';
                $category = $s;
            }
            else {
                $shared = '';
            }
            */

            //
            // original code:
            //
            //   $q[$single] = "id IN (SELECT c.webcamsId FROM webcams$multi c, webcams w WHERE c.name='$category' AND c.webcamsId=w.id AND w.status!='o')";
            //
            // code using webcamsTags and webcamsCategories tables:
            //
            //   $q[$single] = "id IN (SELECT webcamsId FROM webcams$multi WHERE name='$category')";
            //
            // code as above but using the shared redis prefix if any:
            //
            //   $q[$single] = "id IN (SELECT webcamsId FROM ${shared}webcams$multi WHERE name='$category')";
            //
            // code using list of model ids from redis:
            // also need to add on extra optional filtering of gender
            // categories['test'] =       [ test, girls, f ]
            //
            // $r = \Avos\Redis\get("^$p/webcams/$single/$s");
            //
            // $ids = [];
            // foreach ($r as $m) {
            //     $ids[] = $m['i'];
            // }
            //
            // $q[$single] = "id IN (" . implode(',', $ids) . ")";
            //
		}
    }
	
	//
	// search videos by tag
	//

	if (isset($_AVOS['page']['type']) && $_AVOS['page']['type'] == 'videos' && isset($q['tag'])) {
		$tag = $q['tag'];
		$q['tag'] = "id IN (SELECT videosId FROM videosTags WHERE name='$tag')";
	}

	//
	// search pages by tag
	//

	if (isset($_AVOS['page']['type']) && $_AVOS['page']['type'] == 'posts' && isset($q['tag'])) {
		$tag = $q['tag'];
		$q['tag'] = "id IN (SELECT postsId FROM postsTags WHERE name='$tag')";
	}

    //
    // webcams
    //

    if (isset($_AVOS['page']['type']) && $_AVOS['page']['type'] == 'webcams') {

    	//
    	// if no status defined add in "not offline" for type='webcams'
    	//

		if (!isset($q['status']) && !isset($q['category'])) {
			$q['status'] = '!o';
		}

		if (!isset($q['site']) && !isset($q['category']) && !isset($q['tag']) && !empty($sitesFilter)) {
			$q['site'] = $sitesFilter;
		}

        //
        // if a site is defined but not within the sitesFilter then remove it
        // todo: doesnt work, need to explode and also check if site= starts with !
        //

        /*
        if (!empty($sitesFilter)) {
	    	if (strpos($sitesFilter, $q['site']) === false) {
	    		$q['site'] = "xx";
	    	}
        }
         */

        //
        // if we have tags= set to a single item then append #
        //

        if (!empty($q['tags']) && !preg_match('![#\*]!', $q['tags'])) {
            $q['tags'] .= '#';
        }

        //
        // if we have kw= set to a single item then append *
        //

        if (!empty($q['kw']) && !preg_match('![#\*]!', $q['kw'])) {
            $q['kw'] .= '*';
        }
	}

	//
	// if we have a key of 'site' convert to its short form
	//

	if (isset($q['site'])) {
		$q['site'] = \Avos\Scaffold\shortSiteName($q['site']);
	}

	//
	// iterate round the filter converting to sql
	//

    foreach ($q as $k => $v) {

		// skip any arg starting with an underscore

		if ($k[0] == '_') {
			continue;
		}

		$op = '';
		$islike = false;

		// category

		if ($k == 'category') {
            $k = '';
			// $v is already correct
		}

		// tag

		elseif ($k == 'tag') {
			$k = '';
			// $v is already correct
		}

		// a -> != "a"
		// a -> NOT IN (a,b,c)

		elseif (isset($v[0]) && $v[0] == '!') {
			if (strpos($v, '|') !== false) {
				$op = ' NOT IN ';
				$v = str_replace('|', '","', substr($v, 1));
				$v = '("' . $v . '")';
			}
			else {
				$op = ' != ';
				$v = "'" . ltrim($v, '!') . "'";
			}
		}

		// a|b|c -> IN ("a","b","c")

		elseif (strpos($v, '|') !== false) {
			$op = ' IN ';
			$v = str_replace('|', '","', $v);
			$v = '("' . $v . '")';
		}

		// ~a -> LIKE "%a%"

        elseif (strpos($v, '~') !== false) {
            $islike = true;
            $v = $k . ' LIKE "%' . str_replace('~', '', $v) . '%"';
		}

		// a*b -> x LIKE '%a%' OR x LIKE '%b%'

        elseif (strpos($v, '*') !== false || substr($k, 0, 2) == 'kw') {
			$islike = true;

            // special case for 'kw'

            if (substr($k, 0, 2) == 'kw') {
                $islike = true;
                $a = array();

                foreach (explode('*', $v) as $vv) {
                    if (!empty($vv)) {
                        $kw = \Avos\Database\quoteLike($vv);
                        switch ($_AVOS['page']['type']) {
                            case 'webcams':
                                $a[] = "(name LIKE $kw OR nickname LIKE $kw OR subject LIKE $kw OR tags LIKE $kw OR location LIKE $kw OR languages LIKE $kw OR body LIKE $kw)";
                                break;

                            case 'videos':
                                $a[] = "(title LIKE $kw OR tags LIKE $kw OR description LIKE $kw)";
                                break;

                            case 'posts':
                                $a[] = "(title LIKE $kw OR tags LIKE $kw OR body LIKE $kw)";
                                break;
                        }
                    }
                }

                $v = '(' . implode(' OR ', $a) . ')';
            }
            else {

                // get here for 'tags'

                $vv = array();

                foreach (explode('*', $v) as $t) {
                    if (!empty($t)) {
                        $vv[] = "$k LIKE \"%$t%\"";
                    }
                }

                $v = '(' . implode(' OR ', $vv) . ')';
            }

		}

		// a#b -> x LIKE '%,a,%' OR x LIKE '%,b,%'

		elseif (strpos($v, '#') !== false) {
			$islike = true;
			$vv = array();

            foreach (explode('#', $v) as $t) {
                if (!empty($t)) {
                    if (substr($k, 0, 2) == 'kw') {
                        switch ($_AVOS['page']['type']) {
                            case 'webcams':
                                $vv[] = "(name LIKE \"%,$t,%\" OR nickname LIKE \"%,$t,%\" OR subject LIKE \"%,$t,%\" OR tags LIKE \"%,$t,%\" OR location LIKE \"%,$t,%\" OR languages LIKE \"%,$t,%\" OR body LIKE \"%,$t,%\")";
                                break;

                            case 'videos':
                                $vv[] = "(title LIKE \"%,$t,%\" OR tags LIKE \"%,$t,%\" OR description LIKE \"%,$t,%\")";
                                break;
                        }
                    }
                    else {
                        # tags=
                        $vv[] = "$k LIKE \"%,$t,%\"";
                    }
                }
			}

			$v = '(' . implode(' OR ', $vv) . ')';
		}

		// a|b -> x IN ('a', 'b')

		elseif (strpos($v, '|') !== false) {
			echo "TODO: query2sql() a=b|c|d";exit;
		}

		// a < b
		// non-standard so at this point we have something like k="a<b" and v=""

		elseif (strpos($k, '<') !== false) {
			if (preg_match('!(.*?)<(\d+)(d?)!', $k, $c)) {
				$k = $c[1];
				$op = '<';
				$v = $c[2];
				#if ($dbg) { echo "case '<': c="; print_r($c);}

				// special case for addedon

				if (!empty($c[3])) {
					// 30d means 30 'days ago' so subtract 30 days from current timestamp
					$v = time() - ($v * 86400);
					$op = '>';
				}
			}
		}

		// a > b

		elseif (strpos($k, '>') !== false) {
			if (preg_match('!(.*)<(.*?)!', $k, $c)) {
				$k = $c[1];
				$op = '>';
				$v = $c[2];
			}
        }

        // ..a
        // a..
        // a..b

        elseif (strpos($v, '..') !== false) {

            if (preg_match('!^(\d+)?(d?)\.\.(\d+)?(d?)$!', $v, $c)) {

                $min = $c[1];
                $max = $c[3];

                if (!empty($c[2])) {
                    $min = time() - ($min * 86400);
                }

                if (!empty($c[4])) {
                    $max = time() - ($max * 86400);
                }

                if (empty($min)) {
                    $op = ($k == 'addedon') ? '>' : '<=';
                    $v = $max;
                }
                elseif (empty($max)) {
                    $op = ($k == 'addedon') ? '<' : '>=';
                    $v = $min;
                }
                else {
                    if ($k == 'addedon') {
                        $t = $min;
                        $min = $max;
                        $max = $t;
                    }

                    $islike = true;
                    $v = "$k >= $min AND $k <= $max";
                }
            }
        }

		// a -> "a"

		else {
            $op = ' = ';
            $v = '"' . $v . '"';
		}

		// append the item

		if ($v != '') {
			if ($islike == false) {
				$sql[] = "$k$op$v";
			}
			else {
				$sql[] = $v;
			}
		}
	}

	$sql = implode(" AND ", $sql);
    if ($dbg) { echo "$sql\n-->\n\n"; }

	// %X might be a special character so escape X
	#$sql = str_replace('%', '%\\', $sql);
    #$sql = str_replace('\\"', '"', $sql);

    if (isset($_GET['debug'])) echo "<p><b>SQL</b> $sql</p>\n";
	return $sql;
}

function longStatus($s)
{
	$map = array('o' => 'offline', 'a' => 'away', 'f' => 'freechat', 'g' => 'group', 'p' => 'private');
	
	if (isset($map[$s])) {
		return $map[$s];
	}
	else {
		return $s;
	}
}

function normalize_value($value, $options, $default = 'other')
{
    if (isset($options[$value]))
    {   
        return $options[$value];
    }
    else {
        return $default;
    }
}

function list_dirs($dir)
{
	$di = new \DirectoryIterator($dir);
	$dirs = array();

	foreach ($di as $finfo)
	{
		if ($finfo->isDot()) continue;
		if ($finfo->isDir()) array_push($dirs, $finfo->getFilename());
	}

	sort($dirs);

	return $dirs;
}

function list_files($dir)
{
	if (!file_exists($dir)) {
		return array();
	}

	$di = new \DirectoryIterator($dir);
	$dirs = array();

	foreach ($di as $finfo)
	{
		$filename = $finfo->getFilename();
		if (substr($filename, 0, 1) == '.') continue;
		if (!$finfo->isDir()) array_push($dirs, $filename);
	}

	sort($dirs);

	return $dirs;
}

function yymmddToDate($date)
{
    if (preg_match('!(\d\d)(\d\d)(\d\d)!', $date, $c)) {
        $ret = "20$c[1]/$c[2]";
        if (!empty($c[3])) {
            $ret .= "/$c[3]";
        }
        return $ret;
    }
    else {
        return '';
    }
}

function yearsMonthsDaysKeys($start, $end, $monthsOnly=false)
{
    $epoch = $starte = strtotime(yymmddToDate($start));
    $ende = strtotime(yymmddToDate($end));
    $dates = [];

    while (($epoch += 86400) < $ende) {
        $dates []= date('ymd', $epoch);
    }

    if ($monthsOnly) {
        $months = [];
        foreach ($dates as $d) {
            $months[ substr($d, 0, 4) ] = 1;
        }
        $months = array_keys($months);
        array_shift($months);
        return $months;
    }
    else {
        return $dates;
    }
}

function yearsMonthsKeys($months, $days=false)
{
	$keys = array();
	$monthEnd = 0 - $months;
	$monthIter = 0;
	$monthNow = date('n');
	$year = date('y');

	while ($monthIter-- >= $monthEnd) {

		if ($days) {
			for ($i=1; $i<=31; $i++) {
				$keys[] = sprintf("%02d%02d%02d", $year, $monthNow, $i);
			}
		}
		else {
			$keys[] = sprintf("%02d%02d", $year, $monthNow);
		}

		$monthNow--;

		if ($monthNow == 0) {
			$monthNow = 12;
			$year--;
		}
	}

	sort($keys);

	return $keys;
}

function url2array($args)
{
	$tmp = array();

	foreach (explode('&', $args) as $arg)
	{
		//list($k, $v) = explode('=', $arg);
		$tmp_kv = explode('=', $arg);
		$k = isset($tmp_kv[0]) ? $tmp_kv[0] : '';
		$v = isset($tmp_kv[1]) ? $tmp_kv[1] : '';

		// expand foo[] to array

		if (preg_match('!(.*)\[\]$!', $k, $cap))
		{
			if (is_array($tmp[$cap[1]]))
			{
				$tmp[$cap[1]][] = $v;
			}
			else {
				$tmp[$cap[1]] = array($v);
			}
		}
		else {
			$tmp[$k] = $v;
		}
	}

	return $tmp;
}

function array2url($args)
{
	$tmp = array();
	foreach ($args as $k => $v)
	{
		if (is_array($v))
		{
			$tmp[] = "$k=" . implode('|', $v);
		}
		elseif (!empty($k)) {
			$tmp[] = "$k=$v";
		}
	}
	return implode('&', $tmp);
}

function readSpreadsheet($file)
{
    $spready = [];

    if (!file_exists($file)) {
        return [];
    }

    //
    // convert either xslx or csv to array of arrays
    //

    if (preg_match('!\.xlsx$!i', $file)) {
        if ($xlsx = \SimpleXLSX::parseFile($file)) {
            $spready = $xlsx->rows();
        }
        else {
            echo SimpleXLSX::parseError();
        }
    }

    else {
        $fh = fopen($file, 'r');
        while ($row = fgetcsv($fh)) {
            $spready[] = $row;
        }
        fclose($fh);
    }

    return $spready;
}

// Call this at each point of interest, passing a descriptive string
function profFlag($str)
{
    global $prof_timing, $prof_names;
    $prof_timing[] = microtime(true);
    $prof_names[] = $str;
}

// Call this when you're done and want to see the results
function profPrint()
{
    global $prof_timing, $prof_names;
    $size = count($prof_timing);

    echo "<h3>Performance Metrics</h3>";

    for($i=0;$i<$size - 1; $i++)
    {
        $time =  $prof_timing[$i+1]-$prof_timing[$i];

        $css = 'color:red';
        if ($time < 0.01) $css = 'color:#444';
        elseif ($time > 0.1) $css = 'color:red;font-weight:bold';

        echo "<b>{$prof_names[$i]}</b>";
        echo "<span style='$css'>" . sprintf("&nbsp;&nbsp;&nbsp;%f</span><br>", $time);
    }
    echo "<b>{$prof_names[$size-1]}</b><br>";

	printf("<b>page rendered in</b> %f seconds", end($prof_timing) - $prof_timing[0]);
}

function shuffle_assoc(&$array) 
{
    $keys = array_keys($array);
    shuffle($keys);
    $new = [];

    foreach($keys as $key) {
        $new[$key] = $array[$key];
    }

    $array = $new;
    return true;
}

function truncate($str, $len) 
{
    if (strlen($str) > $len) {
        $arr = explode("\n", wordwrap($str, $len-4));
        return array_shift($arr) . "...";
    }

    return $str;
}

function array_psplice(&$array, $offset = 0, $length = 1) 
{
	$return = array_slice($array, $offset, $length, true);

    foreach ($return as $key => $value) {
		unset($array[$key]);
	}

	return $return;
}

function rpcKeyHash($domain, $key)
{
    return md5($domain.'avos'.$key.'avos'.$domain);
}

function url2domain($url)
{
    return preg_replace('!(https://)?(www\.)?(.*\w)?/?.*!', '$3', $url);
}

function http_fetch($url)
{
    $start = microtime(true);
    $resp = @file_get_contents($url);

    $code = 0;
    if (preg_match('#HTTP/\d+\.\d+\s+(\d{3})#', $http_response_header[0], $c)) {
        $code = $c[1];
    }

    return [$resp, $code, microtime(true) - $start];
}

function is_sentences($s, $delim = '\.!')
{
    return count(split_sentences($s, $delim)) > 1;
}

function split_sentences($s, $delim = '\.!')
{
    $ret = [];

    foreach (preg_split('/(?<=[^\d][' . $delim . '])/', $s) as $r) {
        $trimmed = trim(trim($r));

        if (!empty($trimmed)) {
            $ret[] = $trimmed;
        }
    }

    return $ret;

    /*
    $save = '';

    foreach (preg_split('/((?:[^\d])(?:[' . $delim . ']))/', $s, 0, PREG_SPLIT_DELIM_CAPTURE) as $i => $r) {
        if ($i % 2 == 0) {
            $save = $r;
        }
        else {
            $ret[] = trim($save.$r);
            $save = '';
        }
    }

    return $ret;
    */
}

function http_request($url, $method = 'GET', $data = NULL, $headers = NULL, $followRedirects = true)
{
	$header = '';
	$body = '';
	$status = '';

	# in compliance with the RFC 2616 post data will not redirected
	$method = strtoupper($method);
	$url_parsed = @parse_url($url);
	if (!@$url_parsed['scheme']) $url_parsed = @parse_url('http://'.$url);
	extract($url_parsed);

	if(is_array($data))
	{
		$ampersand = '';
		$temp = NULL;

		foreach($data as $k => $v)
		{
			$temp .= $ampersand.urlencode($k).'='.urlencode($v);
			$ampersand = '&';
		}

		$data = $temp;
	}

	if(!@$port) $port = 80;
	if(!@$path) $path = '/';
	if(($method == 'GET') and ($data)) $query = (@$query)?'&'.$data:'?'.$data;
	if(@$query) $path .= '?'.$query;

	$out = "$method $path HTTP/1.0\r\n";
	$out .= "Host: $host\r\n";

	if($method == 'POST')
	{
		$out .= "Content-type: text/xml\r\n";
		$out .= "Content-length: " . @strlen($data) . "\r\n";
	}

	$out .= (@$headers)?$headers:'';
	$out .= "Connection: Close\r\n\r\n";

	if($method == 'POST') $out .= $data."\r\n";

	if(!$fp = @fsockopen($host, $port, $es, $en, 5)){
	return false;
	}
	fwrite($fp, $out);
	while (!feof($fp)) {
		$s = fgets($fp, 128);
		//echo "----------- $s";
	if ( $s == "\r\n" ) {
	$foundBody = true;
	continue;
	}
	if ( isset($foundBody) && !empty($foundBody) ) {
	$body .= $s;
	} else {
	
	//echo $s;
	
	if(($method != 'POST') and ($followRedirects) and (preg_match('/^Location:(.*)/i', $s, $matches) != false) )
	{
	fclose($fp);
	return http_request( trim($matches[1]) );
	}
	$header .= $s;
	if(preg_match('@HTTP[/]1[.][01x][\s]{1,}([1-5][01][0-9])[\s].*$@', $s, $matches))
	{
	$status = trim($matches[1]);
	}
	}
	}
	fclose($fp);

	return array('head' => trim($header), 'body' => preg_replace('/[\x00-\x1F\x80-\xFF]/', '', trim($body)), 'status' => $status);
}

function utf8_filter(string $value): string{
	return preg_replace('/(?:\\\\u[\pL\p{Zs}])+/', '', $value);
    //return preg_replace('/[^[:print:]\n]/u', '', mb_convert_encoding($value, 'UTF-8', 'UTF-8'));
}

function convert_ascii($string) 
{ 
  // Replace Single Curly Quotes
  $search[]  = chr(226).chr(128).chr(152);
  $replace[] = "'";
  $search[]  = chr(226).chr(128).chr(153);
  $replace[] = "'";

  // Replace Smart Double Curly Quotes
  $search[]  = chr(226).chr(128).chr(156);
  $replace[] = '"';
  $search[]  = chr(226).chr(128).chr(157);
  $replace[] = '"';

  // Replace En Dash
  $search[]  = chr(226).chr(128).chr(147);
  $replace[] = '--';

  // Replace Em Dash
  $search[]  = chr(226).chr(128).chr(148);
  $replace[] = '---';

  // Replace Bullet
  $search[]  = chr(226).chr(128).chr(162);
  $replace[] = '*';

  // Replace Middle Dot
  $search[]  = chr(194).chr(183);
  $replace[] = '*';

  // Replace Ellipsis with three consecutive dots
  $search[]  = chr(226).chr(128).chr(166);
  $replace[] = '...';

  // Apply Replacements
  $string = str_replace($search, $replace, $string);

  // Remove any non-ASCII Characters
  $string = preg_replace("/[^\x01-\x7F]/","", $string);

  return $string; 
}

function countryCode($c)
{
	$code2name = array (
		'AF' => 'Afghanistan',
		'AX' => 'Aland Islands',
		'AL' => 'Albania',
		'DZ' => 'Algeria',
		'AS' => 'American Samoa',
		'AD' => 'Andorra',
		'AO' => 'Angola',
		'AI' => 'Anguilla',
		'AQ' => 'Antarctica',
		'AG' => 'Antigua And Barbuda',
		'AR' => 'Argentina',
		'AM' => 'Armenia',
		'AW' => 'Aruba',
		'AU' => 'Australia',
		'AT' => 'Austria',
		'AZ' => 'Azerbaijan',
		'BS' => 'Bahamas',
		'BH' => 'Bahrain',
		'BD' => 'Bangladesh',
		'BB' => 'Barbados',
		'BY' => 'Belarus',
		'BE' => 'Belgium',
		'BZ' => 'Belize',
		'BJ' => 'Benin',
		'BM' => 'Bermuda',
		'BT' => 'Bhutan',
		'BO' => 'Bolivia',
		'BA' => 'Bosnia And Herzegovina',
		'BW' => 'Botswana',
		'BV' => 'Bouvet Island',
		'BR' => 'Brazil',
		'IO' => 'British Indian Ocean Territory',
		'BN' => 'Brunei Darussalam',
		'BG' => 'Bulgaria',
		'BF' => 'Burkina Faso',
		'BI' => 'Burundi',
		'KH' => 'Cambodia',
		'CM' => 'Cameroon',
		'CA' => 'Canada',
		'CV' => 'Cape Verde',
		'KY' => 'Cayman Islands',
		'CF' => 'Central African Republic',
		'TD' => 'Chad',
		'CL' => 'Chile',
		'CN' => 'China',
		'CX' => 'Christmas Island',
		'CC' => 'Cocos (Keeling) Islands',
		'CO' => 'Colombia',
		'KM' => 'Comoros',
		'CG' => 'Congo',
		'CD' => 'Congo, Democratic Republic',
		'CK' => 'Cook Islands',
		'CR' => 'Costa Rica',
		'CI' => 'Cote D\'Ivoire',
		'HR' => 'Croatia',
		'CU' => 'Cuba',
		'CY' => 'Cyprus',
		'CZ' => 'Czech Republic',
		'DK' => 'Denmark',
		'DJ' => 'Djibouti',
		'DM' => 'Dominica',
		'DO' => 'Dominican Republic',
		'EC' => 'Ecuador',
		'EG' => 'Egypt',
		'SV' => 'El Salvador',
		'GQ' => 'Equatorial Guinea',
		'ER' => 'Eritrea',
		'EE' => 'Estonia',
		'ET' => 'Ethiopia',
		'FK' => 'Falkland Islands (Malvinas)',
		'FO' => 'Faroe Islands',
		'FJ' => 'Fiji',
		'FI' => 'Finland',
		'FR' => 'France',
		'GF' => 'French Guiana',
		'PF' => 'French Polynesia',
		'TF' => 'French Southern Territories',
		'GA' => 'Gabon',
		'GM' => 'Gambia',
		'GE' => 'Georgia',
		'DE' => 'Germany',
		'GH' => 'Ghana',
		'GI' => 'Gibraltar',
		'GR' => 'Greece',
		'GL' => 'Greenland',
		'GD' => 'Grenada',
		'GP' => 'Guadeloupe',
		'GU' => 'Guam',
		'GT' => 'Guatemala',
		'GG' => 'Guernsey',
		'GN' => 'Guinea',
		'GW' => 'Guinea-Bissau',
		'GY' => 'Guyana',
		'HT' => 'Haiti',
		'HM' => 'Heard Island & Mcdonald Islands',
		'VA' => 'Holy See (Vatican City State)',
		'HN' => 'Honduras',
		'HK' => 'Hong Kong',
		'HU' => 'Hungary',
		'IS' => 'Iceland',
		'IN' => 'India',
		'ID' => 'Indonesia',
		'IR' => 'Iran, Islamic Republic Of',
		'IQ' => 'Iraq',
		'IE' => 'Ireland',
		'IM' => 'Isle Of Man',
		'IL' => 'Israel',
		'IT' => 'Italy',
		'JM' => 'Jamaica',
		'JP' => 'Japan',
		'JE' => 'Jersey',
		'JO' => 'Jordan',
		'KZ' => 'Kazakhstan',
		'KE' => 'Kenya',
		'KI' => 'Kiribati',
		'KR' => 'Korea',
		'KW' => 'Kuwait',
		'KG' => 'Kyrgyzstan',
		'LA' => 'Lao People\'s Democratic Republic',
		'LV' => 'Latvia',
		'LB' => 'Lebanon',
		'LS' => 'Lesotho',
		'LR' => 'Liberia',
		'LY' => 'Libyan Arab Jamahiriya',
		'LI' => 'Liechtenstein',
		'LT' => 'Lithuania',
		'LU' => 'Luxembourg',
		'MO' => 'Macao',
		'MK' => 'Macedonia',
		'MG' => 'Madagascar',
		'MW' => 'Malawi',
		'MY' => 'Malaysia',
		'MV' => 'Maldives',
		'ML' => 'Mali',
		'MT' => 'Malta',
		'MH' => 'Marshall Islands',
		'MQ' => 'Martinique',
		'MR' => 'Mauritania',
		'MU' => 'Mauritius',
		'YT' => 'Mayotte',
		'MX' => 'Mexico',
		'FM' => 'Micronesia, Federated States Of',
		'MD' => 'Moldova',
		'MC' => 'Monaco',
		'MN' => 'Mongolia',
		'ME' => 'Montenegro',
		'MS' => 'Montserrat',
		'MA' => 'Morocco',
		'MZ' => 'Mozambique',
		'MM' => 'Myanmar',
		'NA' => 'Namibia',
		'NR' => 'Nauru',
		'NP' => 'Nepal',
		'NL' => 'Netherlands',
		'AN' => 'Netherlands Antilles',
		'NC' => 'New Caledonia',
		'NZ' => 'New Zealand',
		'NI' => 'Nicaragua',
		'NE' => 'Niger',
		'NG' => 'Nigeria',
		'NU' => 'Niue',
		'NF' => 'Norfolk Island',
		'MP' => 'Northern Mariana Islands',
		'NO' => 'Norway',
		'OM' => 'Oman',
		'PK' => 'Pakistan',
		'PW' => 'Palau',
		'PS' => 'Palestinian Territory, Occupied',
		'PA' => 'Panama',
		'PG' => 'Papua New Guinea',
		'PY' => 'Paraguay',
		'PE' => 'Peru',
		'PH' => 'Philippines',
		'PN' => 'Pitcairn',
		'PL' => 'Poland',
		'PT' => 'Portugal',
		'PR' => 'Puerto Rico',
		'QA' => 'Qatar',
		'RE' => 'Reunion',
		'RO' => 'Romania',
		'RU' => 'Russian Federation',
		'RW' => 'Rwanda',
		'BL' => 'Saint Barthelemy',
		'SH' => 'Saint Helena',
		'KN' => 'Saint Kitts And Nevis',
		'LC' => 'Saint Lucia',
		'MF' => 'Saint Martin',
		'PM' => 'Saint Pierre And Miquelon',
		'VC' => 'Saint Vincent And Grenadines',
		'WS' => 'Samoa',
		'SM' => 'San Marino',
		'ST' => 'Sao Tome And Principe',
		'SA' => 'Saudi Arabia',
		'SN' => 'Senegal',
		'RS' => 'Serbia',
		'SC' => 'Seychelles',
		'SL' => 'Sierra Leone',
		'SG' => 'Singapore',
		'SK' => 'Slovakia',
		'SI' => 'Slovenia',
		'SB' => 'Solomon Islands',
		'SO' => 'Somalia',
		'ZA' => 'South Africa',
		'GS' => 'South Georgia And Sandwich Isl.',
		'ES' => 'Spain',
		'LK' => 'Sri Lanka',
		'SD' => 'Sudan',
		'SR' => 'Suriname',
		'SJ' => 'Svalbard And Jan Mayen',
		'SZ' => 'Swaziland',
		'SE' => 'Sweden',
		'CH' => 'Switzerland',
		'SY' => 'Syrian Arab Republic',
		'TW' => 'Taiwan',
		'TJ' => 'Tajikistan',
		'TZ' => 'Tanzania',
		'TH' => 'Thailand',
		'TL' => 'Timor-Leste',
		'TG' => 'Togo',
		'TK' => 'Tokelau',
		'TO' => 'Tonga',
		'TT' => 'Trinidad And Tobago',
		'TN' => 'Tunisia',
		'TR' => 'Turkey',
		'TM' => 'Turkmenistan',
		'TC' => 'Turks And Caicos Islands',
		'TV' => 'Tuvalu',
		'UG' => 'Uganda',
		'UA' => 'Ukraine',
		'AE' => 'United Arab Emirates',
		'GB' => 'United Kingdom',
		'US' => 'United States',
		'UM' => 'United States Outlying Islands',
		'UY' => 'Uruguay',
		'UZ' => 'Uzbekistan',
		'VU' => 'Vanuatu',
		'VE' => 'Venezuela',
		'VN' => 'Viet Nam',
		'VG' => 'Virgin Islands, British',
		'VI' => 'Virgin Islands, U.S.',
		'WF' => 'Wallis And Futuna',
		'EH' => 'Western Sahara',
		'YE' => 'Yemen',
		'ZM' => 'Zambia',
		'ZW' => 'Zimbabwe',
	);

    $name2code = array();
    foreach ($code2name as $code => $name) {
        $name2code[$name] = $code;
    }

    $cl = strtolower($c);
    $cu = strtoupper($c);

    if (strlen($c) > 2) {
        if (!empty($name2code[$c])) {
            return $name2code[$c];
        }
        elseif (!empty($name2code[$cl])) {
            return $name2code[$cl];
        }
        else {
            return $c;
        }
    }
    else {
        if (!empty($code2name[$c])) {
            return $code2name[$c];
        }
        elseif (!empty($code2name[$cu])) {
            return $code2name[$cu];
        }
        elseif (!empty($code2name[$cl])) {
            return $code2name[$cl];
        }
        else {
            return $c;
        }
    }
}

function languageCode($c)
{
	$code2name = array (
        'ab' => 'Abkhazian',
        'aa' => 'Afar',
        'af' => 'Afrikaans',
        'ak' => 'Akan',
        'sq' => 'Albanian',
        'am' => 'Amharic',
        'ar' => 'Arabic',
        'an' => 'Aragonese',
        'hy' => 'Armenian',
        'as' => 'Assamese',
        'av' => 'Avaric',
        'ae' => 'Avestan',
        'ay' => 'Aymara',
        'az' => 'Azerbaijani',
        'bm' => 'Bambara',
        'ba' => 'Bashkir',
        'eu' => 'Basque',
        'be' => 'Belarusian',
        'bn' => 'Bengali',
        'bh' => 'Bihari languages',
        'bi' => 'Bislama',
        'bs' => 'Bosnian',
        'br' => 'Breton',
        'bg' => 'Bulgarian',
        'my' => 'Burmese',
        'ca' => 'Catalan, Valencian',
        'km' => 'Central Khmer',
        'ch' => 'Chamorro',
        'ce' => 'Chechen',
        'ny' => 'Chichewa, Chewa, Nyanja',
        'zh' => 'Chinese',
        'cu' => 'Church Slavonic, Old Bulgarian, Old Church Slavonic',
        'cv' => 'Chuvash',
        'kw' => 'Cornish',
        'co' => 'Corsican',
        'cr' => 'Cree',
        'hr' => 'Croatian',
        'cs' => 'Czech',
        'da' => 'Danish',
        'dv' => 'Divehi, Dhivehi, Maldivian',
        'nl' => 'Dutch, Flemish',
        'dz' => 'Dzongkha',
        'en' => 'English',
        'eo' => 'Esperanto',
        'et' => 'Estonian',
        'ee' => 'Ewe',
        'fo' => 'Faroese',
        'fj' => 'Fijian',
        'fi' => 'Finnish',
        'fr' => 'French',
        'ff' => 'Fulah',
        'gd' => 'Gaelic, Scottish Gaelic',
        'gl' => 'Galician',
        'lg' => 'Ganda',
        'ka' => 'Georgian',
        'de' => 'German',
        'ki' => 'Gikuyu, Kikuyu',
        'el' => 'Greek (Modern)',
        'kl' => 'Greenlandic, Kalaallisut',
        'gn' => 'Guarani',
        'gu' => 'Gujarati',
        'ht' => 'Haitian, Haitian Creole',
        'ha' => 'Hausa',
        'he' => 'Hebrew',
        'hz' => 'Herero',
        'hi' => 'Hindi',
        'ho' => 'Hiri Motu',
        'hu' => 'Hungarian',
        'is' => 'Icelandic',
        'io' => 'Ido',
        'ig' => 'Igbo',
        'id' => 'Indonesian',
        'ia' => 'Interlingua (International Auxiliary Language Association)',
        'ie' => 'Interlingue',
        'iu' => 'Inuktitut',
        'ik' => 'Inupiaq',
        'ga' => 'Irish',
        'it' => 'Italian',
        'ja' => 'Japanese',
        'jv' => 'Javanese',
        'kn' => 'Kannada',
        'kr' => 'Kanuri',
        'ks' => 'Kashmiri',
        'kk' => 'Kazakh',
        'rw' => 'Kinyarwanda',
        'kv' => 'Komi',
        'kg' => 'Kongo',
        'ko' => 'Korean',
        'kj' => 'Kwanyama, Kuanyama',
        'ku' => 'Kurdish',
        'ky' => 'Kyrgyz',
        'lo' => 'Lao',
        'la' => 'Latin',
        'lv' => 'Latvian',
        'lb' => 'Letzeburgesch, Luxembourgish',
        'li' => 'Limburgish, Limburgan, Limburger',
        'ln' => 'Lingala',
        'lt' => 'Lithuanian',
        'lu' => 'Luba-Katanga',
        'mk' => 'Macedonian',
        'mg' => 'Malagasy',
        'ms' => 'Malay',
        'ml' => 'Malayalam',
        'mt' => 'Maltese',
        'gv' => 'Manx',
        'mi' => 'Maori',
        'mr' => 'Marathi',
        'mh' => 'Marshallese',
        'ro' => 'Moldovan, Moldavian, Romanian',
        'mn' => 'Mongolian',
        'na' => 'Nauru',
        'nv' => 'Navajo, Navaho',
        'nd' => 'Northern Ndebele',
        'ng' => 'Ndonga',
        'ne' => 'Nepali',
        'se' => 'Northern Sami',
        'no' => 'Norwegian',
        'nb' => 'Norwegian Bokmål',
        'nn' => 'Norwegian Nynorsk',
        'ii' => 'Nuosu, Sichuan Yi',
        'oc' => 'Occitan (post 1500)',
        'oj' => 'Ojibwa',
        'or' => 'Oriya',
        'om' => 'Oromo',
        'os' => 'Ossetian, Ossetic',
        'pi' => 'Pali',
        'pa' => 'Panjabi, Punjabi',
        'ps' => 'Pashto, Pushto',
        'fa' => 'Persian',
        'pl' => 'Polish',
        'pt' => 'Portuguese',
        'qu' => 'Quechua',
        'rm' => 'Romansh',
        'rn' => 'Rundi',
        'ru' => 'Russian',
        'sm' => 'Samoan',
        'sg' => 'Sango',
        'sa' => 'Sanskrit',
        'sc' => 'Sardinian',
        'sr' => 'Serbian',
        'sn' => 'Shona',
        'sd' => 'Sindhi',
        'si' => 'Sinhala, Sinhalese',
        'sk' => 'Slovak',
        'sl' => 'Slovenian',
        'so' => 'Somali',
        'st' => 'Sotho, Southern',
        'nr' => 'South Ndebele',
        'es' => 'Spanish, Castilian',
        'su' => 'Sundanese',
        'sw' => 'Swahili',
        'ss' => 'Swati',
        'sv' => 'Swedish',
        'tl' => 'Tagalog',
        'ty' => 'Tahitian',
        'tg' => 'Tajik',
        'ta' => 'Tamil',
        'tt' => 'Tatar',
        'te' => 'Telugu',
        'th' => 'Thai',
        'bo' => 'Tibetan',
        'ti' => 'Tigrinya',
        'to' => 'Tonga (Tonga Islands)',
        'ts' => 'Tsonga',
        'tn' => 'Tswana',
        'tr' => 'Turkish',
        'tk' => 'Turkmen',
        'tw' => 'Twi',
        'ug' => 'Uighur, Uyghur',
        'uk' => 'Ukrainian',
        'ur' => 'Urdu',
        'uz' => 'Uzbek',
        've' => 'Venda',
        'vi' => 'Vietnamese',
        'vo' => 'Volap_k',
        'wa' => 'Walloon',
        'cy' => 'Welsh',
        'fy' => 'Western Frisian',
        'wo' => 'Wolof',
        'xh' => 'Xhosa',
        'yi' => 'Yiddish',
        'yo' => 'Yoruba',
        'za' => 'Zhuang, Chuang',
        'zu' => 'Zulu'
    );

    $cl = strtolower($c);

    if (!empty($code2name[$cl])) {
        return $code2name[$cl];
    }
    else {
        return $c;
    }
}


function transliterate($textcyr = null, $textlat = null) {
    $cyr = array(
        'ж',  'ч',  'щ',   'ш',  'ю',  'а', 'б', 'в', 'г', 'д', 'е', 'з', 'и', 'й', 'к', 'л', 'м', 'н', 'о', 'п', 'р', 'с', 'т', 'у', 'ф', 'х', 'ц', 'ъ', 'ь', 'я',
        'Ж',  'Ч',  'Щ',   'Ш',  'Ю',  'А', 'Б', 'В', 'Г', 'Д', 'Е', 'З', 'И', 'Й', 'К', 'Л', 'М', 'Н', 'О', 'П', 'Р', 'С', 'Т', 'У', 'Ф', 'Х', 'Ц', 'Ъ', 'Ь', 'Я');

    $lat = array(
        'zh', 'ch', 'sht', 'sh', 'yu', 'a', 'b', 'v', 'g', 'd', 'e', 'z', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'r', 's', 't', 'u', 'f', 'h', 'c', 'y', 'x', 'q',
        'Zh', 'Ch', 'Sht', 'Sh', 'Yu', 'A', 'B', 'V', 'G', 'D', 'E', 'Z', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'R', 'S', 'T', 'U', 'F', 'H', 'c', 'Y', 'X', 'Q');

    if($textcyr) return str_replace($cyr, $lat, $textcyr);
    else if($textlat) return str_replace($lat, $cyr, $textlat);
    else return null;
}

function removeStopWords($input)
{
$commonWords = array('a','able','about','above','abroad','according','accordingly','across','actually','adj','after','afterwards','again','against','ago','ahead','ain\'t','all','allow','allows','almost','alone','along','alongside','already','also','although','always','am','amid','amidst','among','amongst','an','and','another','any','anybody','anyhow','anyone','anything','anyway','anyways','anywhere','apart','appear','appreciate','appropriate','are','aren\'t','around','as','a\'s','aside','ask','asking','associated','at','available','away','awfully','b','back','backward','backwards','be','became','because','become','becomes','becoming','been','before','beforehand','begin','behind','being','believe','below','beside','besides','best','better','between','beyond','both','brief','but','by','c','came','can','cannot','cant','can\'t','caption','cause','causes','certain','certainly','changes','clearly','c\'mon','co','co.','com','come','comes','concerning','consequently','consider','considering','contain','containing','contains','corresponding','could','couldn\'t','course','c\'s','currently','d','dare','daren\'t','definitely','described','despite','did','didn\'t','different','directly','do','does','doesn\'t','doing','done','don\'t','down','downwards','during','e','each','edu','eg','eight','eighty','either','else','elsewhere','end','ending','enough','entirely','especially','et','etc','even','ever','evermore','every','everybody','everyone','everything','everywhere','ex','exactly','example','except','f','fairly','far','farther','few','fewer','fifth','first','five','followed','following','follows','for','forever','former','formerly','forth','forward','found','four','from','further','furthermore','g','get','gets','getting','given','gives','go','goes','going','gone','got','gotten','greetings','h','had','hadn\'t','half','happens','hardly','has','hasn\'t','have','haven\'t','having','he','he\'d','he\'ll','hello','help','hence','her','here','hereafter','hereby','herein','here\'s','hereupon','hers','herself','he\'s','hi','him','himself','his','hither','hopefully','how','howbeit','however','hundred','i','i\'d','ie','if','ignored','i\'ll','i\'m','immediate','in','inasmuch','inc','inc.','indeed','indicate','indicated','indicates','inner','inside','insofar','instead','into','inward','is','isn\'t','it','it\'d','it\'ll','its','it\'s','itself','i\'ve','j','just','k','keep','keeps','kept','know','known','knows','l','last','lately','later','latter','latterly','least','less','lest','let','let\'s','like','liked','likely','likewise','little','look','looking','looks','low','lower','ltd','m','made','mainly','make','makes','many','may','maybe','mayn\'t','me','mean','meantime','meanwhile','merely','might','mightn\'t','mine','minus','miss','more','moreover','most','mostly','mr','mrs','much','must','mustn\'t','my','myself','n','name','namely','nd','near','nearly','necessary','need','needn\'t','needs','neither','never','neverf','neverless','nevertheless','new','next','nine','ninety','no','nobody','non','none','nonetheless','noone','no-one','nor','normally','not','nothing','notwithstanding','novel','now','nowhere','o','obviously','of','off','often','oh','ok','okay','old','on','once','one','ones','one\'s','only','onto','opposite','or','other','others','otherwise','ought','oughtn\'t','our','ours','ourselves','out','outside','over','overall','own','p','particular','particularly','past','per','perhaps','placed','please','plus','possible','presumably','probably','provided','provides','q','que','quite','qv','r','rather','rd','re','really','reasonably','recent','recently','regarding','regardless','regards','relatively','respectively','right','round','s','said','same','saw','say','saying','says','second','secondly','see','seeing','seem','seemed','seeming','seems','seen','self','selves','sensible','sent','serious','seriously','seven','several','shall','shan\'t','she','she\'d','she\'ll','she\'s','should','shouldn\'t','since','six','so','some','somebody','someday','somehow','someone','something','sometime','sometimes','somewhat','somewhere','soon','sorry','specified','specify','specifying','still','sub','such','sup','sure','t','take','taken','taking','tell','tends','th','than','thank','thanks','thanx','that','that\'ll','thats','that\'s','that\'ve','the','their','theirs','them','themselves','then','thence','there','thereafter','thereby','there\'d','therefore','therein','there\'ll','there\'re','theres','there\'s','thereupon','there\'ve','these','they','they\'d','they\'ll','they\'re','they\'ve','thing','things','think','third','thirty','this','thorough','thoroughly','those','though','three','through','throughout','thru','thus','till','to','together','too','took','toward','towards','tried','tries','truly','try','trying','t\'s','twice','two','u','un','under','underneath','undoing','unfortunately','unless','unlike','unlikely','until','unto','up','upon','upwards','us','use','used','useful','uses','using','usually','v','value','various','versus','very','via','viz','vs','w','want','wants','was','wasn\'t','way','we','we\'d','welcome','well','we\'ll','went','were','we\'re','weren\'t','we\'ve','what','whatever','what\'ll','what\'s','what\'ve','when','whence','whenever','where','whereafter','whereas','whereby','wherein','where\'s','whereupon','wherever','whether','which','whichever','while','whilst','whither','who','who\'d','whoever','whole','who\'ll','whom','whomever','who\'s','whose','why','will','willing','wish','with','within','without','wonder','won\'t','would','wouldn\'t','x','y','yes','yet','you','you\'d','you\'ll','your','you\'re','yours','yourself','yourselves','you\'ve','z','zero');

$input = preg_replace('/\b('.implode('|',$commonWords).')\b/','',$input);
return trim(preg_replace('/\s\s+/', ' ', str_replace("\n", " ", $input)));
}


