<?
define('TOP', dirname(dirname(dirname($_SERVER['SCRIPT_FILENAME']))) . '/avos/');
define('TEMPLATES', TOP . 'templates.admin/');

include(TOP . 'bin/bootstrap.php');

checkAuth();

switch ($_GET['a']) {
    case 'sites':
        $resp = listSites();
        break;

    default:
        $resp = [];
        break;
}

//
// output response
//


if (!($json = json_encode(['data' => $resp]))) {
    $json = ['error' => json_last_error_msg()];
}

echo $json;
exit;

//
// list sites on the server
//

function listSites()
{
    $top = dirname(dirname(TOP));
    $dirs = \Avos\Utils\list_dirs($top);
    $redis = \Avos\Redis\handle();
    $sites = [];

    foreach ($dirs as $domain) {
        if (file_exists($top . '/' . $domain . '/avos/config.ini')) {

            $config = parse_ini_file($top . '/' . $domain . '/avos/config.ini', true);

            if (isset($config['site']['serversSkip']) && $config['site']['serversSkip']) {
                continue;
            }

            $sites[$domain] = ['requests' => 0, 'requestsHours' => 0, 'requestsHour' => 0, 'http' => 0, 'pageLoad' => 0, 'config' => $config];

            //
            // see if we have traffic logs
            //

            if (isset($sites[$domain]['config']['site']['logRequestsExpiry'])) {
                $prefix = $sites[$domain]['config']['redis']['prefix'];
                $requests = \Avos\Redis\handle()->lRange("$prefix/requests", 0, -1);
                $sites[$domain]['requests'] = count($requests);
                $sites[$domain]['requestsHours'] = $sites[$domain]['config']['site']['logRequestsExpiry'];

                if ($sites[$domain]['requestsHours']) {
                    $sites[$domain]['requestsHour'] = round($sites[$domain]['requests'] / $sites[$domain]['requestsHours']);
                }
                else {
                    $sites[$domain]['requestsHour'] = 0;
                }
            }

            //
            // fetch site and check time and response code negating internet lag
            //

            list($resp, $code, $elapsed) = \Avos\Utils\http_fetch($sites[$domain]['config']['site']['url']);

            $sites[$domain]['http'] = $code;
            $sites[$domain]['pageLoadHome'] = sprintf('%0.3f', $elapsed);

            //
            // check loading a category url
            //

            if (isset($sites[$domain]['config']['urls']['webcamsTag'])) {
                $url = $sites[$domain]['config']['urls']['webcamsTag'];
                $url = trim($url, '[] ');

                if (preg_match('!,\s*/(\S+)!', $url, $c)) {

                    $url = $c[1];
                    $tag = array_key_first($sites[$domain]['config']['webcams']['tags']);
                    $url = str_replace(':tag', $tag, $url);
                    $url = str_replace(':page', '', $url);
                    $url = $sites[$domain]['config']['site']['url'] . $url;

                    list($resp, $code, $elapsed) = \Avos\Utils\http_fetch($url);

                    $sites[$domain]['pageLoadTag'] = sprintf('%0.3f', $elapsed);

                }
            }

            if (!isset($sites[$domain]['pageLoadTag'])) {
                $sites[$domain]['pageLoadTag'] = '';
            }
        }
    }

    return $sites;
}

//
// auth key check
//

function checkAuth()
{
    $rpcKey = getConfig('site', 'rpcKey', '');

    if (empty($rpcKey)) {
        http_response_code(404);
        echo json_encode(['error' => 'missing rpcKey']);
        exit;
    }

    
    $domain = preg_replace('!(https://)?(.*\w)?/?!', '$2', getConfig('site', 'url'));
    $hash = \Avos\Utils\rpcKeyHash($domain, $rpcKey);

    if (empty($_GET['k']) || $_GET['k'] != $hash) {
        http_response_code(404);
        echo json_encode(['error' => 'bad rpcKey']);
        exit;
    }

    if (!isset($_GET['a']) || empty($_GET['a'])) {
        http_response_code(404);
        echo json_encode(['error' => 'bad action']);
        exit;
    }
}
