#  AVOS: The Smart Way to Do Dumb Things (Intelligently)

*Breaking the rules, optimizing the system, and making things work against all odds.*



1. Webcams Functions
    1. [Model Id List to Data](#ExtractWebcamsFields)
    2. [Expanding Models Data from Mysql](#ExpandWebcamsFields)

2. Webcams Statistics 
    1. [All Statistics](#AllStatistics)
    2. [Statistics Summary](#Webcams_Statistics)
    3. [Popular Models List](#Popular_Models_List)
    4. [Trending Models List](#Trending_Models_List)
    5. [Highest Popular Rank](#Highest_Popular_Rank)
   

3. Webcams Add-Ons
    1. [SitesFilter + SitebarSitesFilter](#SitesFilter)
    2. [Promote Models | Events](#Promote)
   
   
4. [Users](#Users)
    1. [Saved Searches](#Saved_Searches)
    2. [Profile Viewing History](#Profile_Viewing_History)

5. [Blogs](#Blogs)
    1. [Add  Blogs](#Add_blogs)
   
6. Avos install Satelits Sites
    1. [Add New Satelit](#Add-Satelit)


******************


---

## Webcams

<div id='ExtractWebcamsFields'/>
### Model Id List to Data

The function `extractWebcamsFields($webcamsIds)` will convert the list of `webcams` id's to an array of hashes containing the models info for display on a grid etc.

#### Example

    $models = (extractWebcamsFields([12345, 12725]);

    [
        [
            [id] => 12345
            [addedOn] => 1631607911
            [updatedOn] => 1741005250
            [scrapedOn] => 1670274856
            [purged] => 0
            [nickname] => riley_sweety
            [externalId] => 0
            [name] => Riley
            [site] => Chaturbate
            [status] => Offline
            ...
        ],
        [
            [id] => 12725
            ...
        ]
    ]





---


<div id='ExpandWebcamsFields'/>
### Expand Mysql Webcams Data

The function `expandWebcamsFields($models)` takes a list of hashes containing webcams data extracted from mysql and expands/adds the following fields:

- imagePath
- siteShortName
- siteShortNameCaps
- site
- status
- gender

Additionally if age is 0 it is converted to the empty string and any data stored in `Redis` in the key `modelId/p` is merged.



---

## Webcams Statistics

<div id='Webcams_Statistics'/>
### Statistics Summary

The function `getStatsSummary()` returns an array containing 3 keys: 

- 'd' (day, last 24 hours online)
- 'm' (month, last 30 days)
- 'a' for all time data.

Each key contains a hash with these keys:

    [viewersTotal] => 330
    [viewersMax] => 48
    [viewersAvg] => 6
    [followersNew] => 704
    [followersMin] => 31477
    [followersMax] => 32181
    [rankMax] => 868
    [rankAvg] => 1541
    [onlineTime] => 371170
    [onlineDays] => 52
    [onlineDaily] => 7137
    [freechatTime] => 350825
    [freechatLongest] => 32041
    [freechatSessions] => 142
    [freechatAvg] => 2470
    [groupTime] => 11704
    [groupLongest] => 4740
    [groupSessions] => 22
    [groupAvg] => 532
    [privateTime] => 8641
    [privateLongest] => 2160
    [privateSessions] => 12
    [privateAvg] => 720

#### functions.php

    function webcamsPopular()
    {
        setTemplateValue('stats', getStatsSummary());
    }



---

<div id='AllStatistics'/>
### All Statistics
The function `getAllStats($webcamsId)` returns all statistics for the specified model both recent data from Redis and all time data that has been purged to SQLite databases on the filesystem. It returns an `array()` with the following keys:

- 'h' (online status timeline by month)

    - eg: 12345/h/2501

- 'd' (summary of daily times in freechat etc)

    - eg: 12345/d/250210

- 'r' (ranks; followers and viewers)

    - eg: 12345/r/2412


    [0] => [

            [e] => 1717206
            [r] => 1515
            [g] => 1050
            [v] => 6
            [f] => 29625
        ]


`[e]` is the timestamp in minutes after `EPOCHSTART` to reduce space, plus cron only runs every minute. to convert to epoch seconds use `$elt['e'] * 60 + EPOCHSTART`



---


<div id='Popular_Models_List'/>
### Popular Models List

Stores the `popularModelsMax` models from the last `popularModelsDays` days, ranking is based on viewers. Run the `popular` cron
to cache the data.

#### config.ini

    [webcams]
    ; these are the defaults if not set
    popularModelsMax = 1000
    popularModelsDays = 30

    [urls]
    webcamsPopular = [ none, /popular/ ]

#### functions.php

    function webcamsPopular()
    {
        setTemplateValue('models', getPopularWebcams());
    }


---


<div id='Trending_Models_List'/>
### Trending Models List

explanation...

#### config.ini

    [webcams]
    ; these are the defaults if not set
    trendingModelsSnapshots = 10
    trendingModelsMax = 1000

    [urls]
    webcamsTrending = [ none, /trending/ ]

#### functions.php

    function webcamsTrending()
    {
        setTemplateValue('models', getTrendingWebcams());
    }

#### webcamsTrending.php

    <? foreach ($models as $m): ?>
        <?= $m['nickname'] ?> trending rank is <?= $m['rank'] ?>
    <? endforeach; ?>



---


<div id='Highest_Popular_Rank'/>
### Highest Popular Rank

If a model has ever ranked in the top `popularModelsMax` models list their best ranking and date is stored in redis under the key `siteprefix/models/modelId/p`

These values are automatically fetched from redis when displaying a profile as `bestRank` and `bestRankOn`

#### webcamsProfile.php

    <? if (!empty($model['bestRank'])): ?>
    Best Ranked <?= $model['bestRank'] ?> on <?= date("F j, Y", $model['bestRankOn']); ?></div>
    <? endif; ?>


---


<div id='Highest_Popular_Rank'/>
### Highest Popular Rank

If a model has ever ranked in the top `popularModelsMax` models list their best ranking and date is stored in redis under the key `siteprefix/models/modelId/p`

These values are automatically fetched from redis when displaying a profile as `bestRank` and `bestRankOn`

#### webcamsProfile.php

    <? if (!empty($model['bestRank'])): ?>
    Best Ranked <?= $model['bestRank'] ?> on <?= date("F j, Y", $model['bestRankOn']); ?></div>
    <? endif; ?>

---



<div id='Users'/>
## Users

The following work whether the user is logged in or not. When logged in data is stored in mysql and in the session if not.

An `array()` is returned from the users functions containing status and data which can automatically returned in json format if required for javascript use
(append `format=json` to a web request to a `[urls]` rule of type `users`)

    define('USERS_STATUS_OK',           1);
    define('USERS_STATUS_ERROR',        0);

    [
        [status] => 1
        [message] => 
        [error] => 0
        [args] => [
            [models] => []
    ]

---

<div id='Saved_Searches'/>
### Saved Searches

`saveSearch()` is automatically called when multiple webcams results are found. `listUsersSearches()` should be called in functions.php

#### config.ini

    [urls]
    usersSearches = [ none, /searches/ ]

#### functions.php

    function usersSearches()
    {
        setTemplateValue('searches', listUsersSearches()['args']['searches']);
    }

---

<div id='Profile_Viewing_History'/>
### Profile Viewing History

`addUsersHistory()` needs to be called in the `[urls]` rule for displaying a profile and `listUsersHistory()` will return a list of model id's that have been viewed.

#### config.ini

    [urls]
    usersHistory =             [ none,   /history/:page ]

#### functions.php

    function webcamsProfile()
    {           
        $model = getTemplateValue('model');
        addUsersHistory($model['id']);
    }

    function usersHistory()
    {
        setTemplateValue('models', listUsersHistory()['args']['models']);
    }


------

<div id='Add_blogs'/>
###  Blogs 

 How to Create and Use a `[blog]` Section in config.ini

#### config.ini
 Create  `URLs` for blog

    [urls] 
    articleIndex                        = [ blogs,           /blog-articles/ ] 
    articleProfile                      = [ blog,            /blog-articles/:Title/:id ]


To configure blog settings, you need to create a` [blog] `section inside your` config.ini` file. 

This section will define various settings for the blog, such as `pagination`, `publishing` `frequency`, `caching`, and `API` keys.


    [blog]
    perPage                = 36  ; Number of blog posts displayed per page
    blogPhotosUrl          = /photosBlog/  ; URL path for blog-related photos
    publishPerDay          = 1  ; One blog post will be published every 5 days (86400 seconds/day ÷ 0.2 = 432000 seconds)
    sitemapsUrlName        = articleProfile  ; Name used for sitemap URLs related to blogs
    blogGalleryUrl         = /photosBlog/GalleryImages  ; URL path for the blog's image gallery
    TinyMCEapi_key         = 'your-apikey-here'  ; API key for TinyMCE editor
    cacheExpiry            = 60  ; Cache expiration time in seconds



----


<div id='SitesFilter'/>
### Webcams Add-Ons

 `sitesFilter` - are the sites that will be  display  in `header` if add you can add `sites` or `sitesDropdown` in `headerMenuItems`
 
 &nbsp;
 &nbsp;
 &nbsp;

 `sitebarSitesFilter` - are the sites that will be  display  in  `sidebar` if add `sitesCamsDropdown` in `sidebarMenuItems`  

#### config.ini

    [webcams]
    sitesFilter                = cb,sc,cs,bc,lj,sr,cr,ff,c4,xl,xc,sm,mf,am,sp
    sitebarSitesFilter         = cb,sc,cs,bc,lj,sr,cr,ff,c4,xl,xc,sm,mf,am,sp

----



<div id='Promote'/>
#### Add  Website | Model | Event  - Promote

 by adding in `[urls]` -  will activate the promote Parteners functions

#### config.ini

    [urls] 
    webcamsParteners                     = [promote,          /live-webcam-model-events/:page] 
    webcamsPartenersDescription          = [promot,           /trending-webcams-live-shows/:siteNames, 'count=300'] 
    



 *****
  by adding in `[promote]` -  will make the sitemaps for promote  Website | Model | Event

    [promote]
    sitemapsUrlName                                                    = webcamsPartenersDescription
    


----



---



<div id='Add-Satelit'/>
### Satelits Sites Install

If you don’t know how to use these commands, `stop right now`. This isn’t a fucking tutorial. Screw it up, and that’s your problem.

    -A. cd /root/bin && ./add_mysql_avos.sh 
    -B. cd /root/bin && ./add_mysql_perm.sh 
    -C. su - username
    -D. cd ~/html/ && cp -Rp oldsite.com newsite.com 
    -E. cd /home/username/html/newsite.com/avos/
    -F. mysqldump -d -u oldsite -poldsite oldsite | mysql -u newsite -pnewsite newsite
    -G. cd /home/username/html/
    -H. cd newsite.com/avos && mv templates templates.safe && ln -s ../../oldsite.com/avos/templates








[Top](#top)